use axum::{
    extract,
    response::{IntoResponse, Json},
    routing::post,
    Router,
};
use serde::{Deserialize, Serialize};
use tokio::net::TcpListener;

#[derive(Serialize)]
struct SomeResponse {
    sum: u32,
}

#[derive(Deserialize)]
pub struct SomePayload {
    // Option is used to make the fields optional
    max_value: Option<u32>,
    min_value: Option<u32>,
}

#[tokio::main]
async fn main() -> Result<(), String> {
    let app = create_routes().await;
    let bind_addr = &"0.0.0.0:2900";
    let listener = TcpListener::bind(bind_addr)
        .await
        .map_err(|e| format!("Failed to parse address: {}", e))?;
    axum::serve(listener, app.into_make_service())
        .await
        .map_err(|e| format!("Server error: {}", e))?;
    Ok(())
}

pub async fn create_routes() -> Router {
    Router::new().route("/api/v1/some", post(some))
}

pub async fn some(
    extract::Json(payload): extract::Json<SomePayload>,
) -> impl IntoResponse {
    let min_value = payload.min_value.unwrap_or_default();
    let max_value = payload.max_value.unwrap_or_default();
    let content = SomeResponse {
        sum: (min_value..=max_value).sum(),
    };
    Json(content)
}
